Attribute VB_Name = "LevyAccountRoutines"
Option Explicit
Public Const errFeeCodeNotFound = vbObjectError + 700 + 1


Public Function GetLevyOpeningBalance(strStudentID As String, dtmStartDate As Date) As Currency
    Dim TotalDebits As Currency
    Dim TotalCredits As Currency
    Dim rstTemp As Recordset
    Dim strSQL As String
    
    'Find Total Debit Entries
    strSQL = "Parameters [Start Date] DateTime; " _
            & "SELECT Sum(Amount) as [Total Debits] " _
            & "FROM [Levy Account Debits] " _
            & "WHERE [Transaction Date] < [Start Date] "
            
    qryQueryObject.SQL = strSQL
    qryQueryObject.Parameters("Start Date") = dtmStartDate
    Set rstTemp = qryQueryObject.OpenRecordset
    
    If IsNull(rstTemp("Total Debits")) Then
        TotalDebits = 0
    Else
        TotalDebits = rstTemp("Total Debits")
    End If
    
    'Find Total Credit Entries
    strSQL = "Parameters [Start Date] DateTime; " _
            & "SELECT Sum(Amount) as [Total Credits] " _
            & "FROM [Levy Account Credits] " _
            & "WHERE [Transaction Date] < [Start Date] "
            
    qryQueryObject.SQL = strSQL
    qryQueryObject.Parameters("Start Date") = dtmStartDate
    Set rstTemp = qryQueryObject.OpenRecordset
    
    If IsNull(rstTemp("Total Credits")) Then
        TotalCredits = 0
    Else
        TotalCredits = rstTemp("Total Credits")
    End If
    
    
    'Return opening balance
    GetLevyOpeningBalance = TotalCredits - TotalDebits
    
End Function

Public Function GetLevyClosingBalance(strStudentID As String, dtmEndDate As Date) As Currency
    Dim TotalDebits As Currency
    Dim TotalCredits As Currency
    Dim rstTemp As Recordset
    Dim strSQL As String
    
    'Find Total Debit Entries
    strSQL = "Parameters [End Date] DateTime, [The Student] String; " _
            & "SELECT Sum(Amount) as [Total Debits] " _
            & "FROM [Levy Account Debits] " _
            & "WHERE [Transaction Date] < [End Date] and [Student ID] = [The Student] "
            
    qryQueryObject.SQL = strSQL
    qryQueryObject.Parameters("End Date") = dtmEndDate + 1
    qryQueryObject.Parameters("The Student") = strStudentID
    Set rstTemp = qryQueryObject.OpenRecordset
    
    If IsNull(rstTemp("Total Debits")) Then
        TotalDebits = 0
    Else
        TotalDebits = rstTemp("Total Debits")
    End If
    
    'Find Total Credit Entries
    strSQL = "Parameters [End Date] DateTime, [The Student] String; " _
            & "SELECT Sum(Amount) as [Total Credits] " _
            & "FROM [Levy Account Credits] " _
            & "WHERE [Transaction Date] < [End Date] and [Student ID] = [The Student] "
            
    qryQueryObject.SQL = strSQL
    qryQueryObject.Parameters("End Date") = dtmEndDate + 1
    qryQueryObject.Parameters("The Student") = strStudentID
    Set rstTemp = qryQueryObject.OpenRecordset
    
    If IsNull(rstTemp("Total Credits")) Then
        TotalCredits = 0
    Else
        TotalCredits = rstTemp("Total Credits")
    End If
    
    'Return opening balance
    GetLevyClosingBalance = TotalCredits - TotalDebits
    
End Function

Public Function GetLevyStatementBodySQL() As String
    Dim strSQL As String
    
    strSQL = "PARAMETERS [The Student] String, " _
            & "         [Start Date] DateTime, " _
            & "         [End Date] DateTime; " _
            & "SELECT [Levy Account Debits].[Transaction Date]," _
            & "       ([Registered Fees].[Fee Name]) as [Description], " _
            & "       ([Levy Account Debits].Amount * -1) As [Amount] " _
            & "FROM [Registered Fees] INNER JOIN [Levy Account Debits] " _
            & "ON [Registered Fees].[Fee Code] = [Levy Account Debits].[Fee Code] " _
            & "WHERE [Levy Account Debits].[Student ID] = [The Student] " _
            & "      AND [Levy Account Debits].[Transaction Date] >= [Start Date] " _
            & "      AND DateValue([Levy Account Debits].[Transaction Date]) <= DateValue([End Date]) " _
            & "UNION SELECT [Transaction Date], " _
            & "             ('Payment - ' &  [Receipt Number]) As [Description], " _
            & "             [Amount] " _
            & "FROM [Levy Account Credits] " _
            & "WHERE [Levy Account Credits].[Student ID] = [The Student] " _
            & "      AND [Levy Account Credits].[Transaction Date] >= [Start Date] " _
            & "      AND DateValue([Levy Account Credits].[Transaction Date]) <= DateValue([End Date]) " _
            & "ORDER BY [Transaction Date];"
    
    GetLevyStatementBodySQL = strSQL
    
            
End Function


Public Function GetChargeArray() As Variant()
    'Returns an array containing fee codes and amounts
    'REQUIRES - frmLevy_Charge.grdCharges
    'THROWS EXCEPTION - errFeeCodeNotFound
    Dim strMsg As String
    Dim strFeeName As String
    Dim lngFeeCode As String
    Dim lngArrayEntries As Long
    Dim varArrayCharges() As Variant
    
    rstRegisteredFees.Index = "Fee Name Index"
    
    'Move to top left corner cell
    frmLevy_Charge.grdCharges.Row = 1
    frmLevy_Charge.grdCharges.Col = 0
    
    While frmLevy_Charge.grdCharges.Text <> ""
        'get fee code
        strFeeName = frmLevy_Charge.grdCharges.Text
        rstRegisteredFees.Seek "=", strFeeName
        
        If rstRegisteredFees.NoMatch Then
            'Throw exception
            strMsg = "Unable to obtain fee information from system. Please try starting " _
                   & "over. No charges have been recorded"
            Err.Raise errFeeCodeNotFound, "Generate Charge Array", strMsg
        Else
            lngFeeCode = rstRegisteredFees("Fee Code")
            
            'Add a blank row to the array
            lngArrayEntries = lngArrayEntries + 1
            ReDim Preserve varArrayCharges(1 To 2, 1 To lngArrayEntries)
            
            'Record fee code
            varArrayCharges(1, lngArrayEntries) = lngFeeCode
            
            'Record fee amount
            frmLevy_Charge.grdCharges.Col = 1
            varArrayCharges(2, lngArrayEntries) = Val(frmLevy_Charge.grdCharges.Text)
            
            'Prepare for next read
            frmLevy_Charge.grdCharges.Col = 0
            frmLevy_Charge.grdCharges.Row = frmLevy_Charge.grdCharges.Row + 1
        End If
    Wend
    
    GetChargeArray = varArrayCharges
    
End Function

